<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 *  トピック情報の取得
 *
 *  @access public
 *  @param  integer $topic_id            トピックID
 */
function db_get_topic_info($topic_id)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT t.topic_id, t.author_name, t.author_email, t.subject, t.category_id, ca.category_name,' .
            ' t.body, t.allow_comment, t.author_password, t.r_datetime, t.m_datetime' .
            ' FROM pbb_topics AS t' .
            ' LEFT JOIN pbb_categories AS ca' .
                ' ON t.category_id = ca.category_id' .
            ' WHERE t.topic_id = ?';
    $params = array(
        intval($topic_id)
    );
    $row = pocket_db_getRow($sql, $params);
    if ($row)
    {
        $row['comment_count'] = db_get_topic_comments_count($row['topic_id']);
    }
    return $row;
}

/**
 *  トピックのコメント件数の取得
 *
 *  @access public
 *  @param  integer $topic_id            トピックID
 */
function db_get_topic_comments_count($topic_id)
{
    $sql = 'SELECT count(topic_comment_id) AS comment_count' .
            ' FROM pbb_topic_comments' .
            ' WHERE topic_id = ?';
    $params = array(
        intval($topic_id)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  最新のトピック一覧情報の取得
 *
 *  @access public
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_topics4latest_topic_list($page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT topic_id, subject, r_datetime, m_datetime' .
            ' FROM pbb_topics' .
            ' ORDER BY r_datetime DESC';
    $list = pocket_db_getAllPage($sql, $page, $page_size);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_topic_comments_count($item['topic_id']);
    }
    return $list;
}

/**
 *  最新のトピック一覧情報（RSS用）の取得
 *
 *  @access public
 *  @param  integer $page_size          ページサイズ
 */
function db_get_topics4latest_topic_rss($page_size = 5)
{
    $sql = 'SELECT topic_id, subject AS title, body AS description, r_datetime' .
            ' FROM pbb_topics' .
            ' ORDER BY r_datetime DESC';
    return pocket_db_getAllPage($sql, 1, $page_size);
}

/**
 *  最新のトピック情報の取得
 *
 *  @access public
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_topics4latest($page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT t.topic_id, t.category_id, ca.category_name, t.author_name, t.subject, t.body, t.r_datetime, t.m_datetime' .
            ' FROM pbb_topics AS t' .
            ' LEFT JOIN pbb_categories AS ca' .
                ' ON t.category_id = ca.category_id' .
            ' ORDER BY t.r_datetime DESC';
    $list = pocket_db_getAllPage($sql, $page, $page_size);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_topic_comments_count($item['topic_id']);
    }
    return $list;
}

/**
 *  カテゴリのトピック情報の取得
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_topics4category($category_id, $page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT topic_id, author_name, subject, body, r_datetime, m_datetime' .
            ' FROM pbb_topics' .
            ' WHERE category_id = ?' .
            ' ORDER BY r_datetime DESC';
    $params = array(
        intval($category_id)
    );
    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_topic_comments_count($item['topic_id']);
    }
    return $list;
}

/**
 *  カテゴリのトピック件数の取得
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 */
function db_get_topics_count4category($category_id)
{
    $sql = 'SELECT count(topic_id)' .
            ' FROM pbb_topics' .
            ' WHERE category_id = ?';
    $params = array(
        intval($category_id)
    );
    return pocket_db_getOne($sql, $params);
}

/**
 *  キーワード検索のトピック情報の取得
 *
 *  @access public
 *  @param  string  $keyword            キーワード
 *  @param  integer $page               ページ番号
 *  @param  integer $page_size          ページサイズ
 */
function db_get_topics4keyword($keyword, $page = 1, $page_size = 5)
{
    //////////////////////////////////////////////
    // MySQL4.0.x 対応のためサブクエリは使用しない
    //////////////////////////////////////////////
    $sql = 'SELECT t.topic_id, t.author_name, t.category_id, ca.category_name, t.subject, t.body, t.r_datetime, t.m_datetime' .
            ' FROM pbb_topics AS t' .
            ' LEFT JOIN pbb_categories AS ca' .
                ' ON t.category_id = ca.category_id' .
            ' WHERE 1 = 1';

    if ($keyword)
    {
        $keyword = str_replace('　', ' ', $keyword);
        $word_list = explode(' ', $keyword);
        foreach ($word_list as $word)
        {
            $sql .= ' AND (t.subject LIKE ? OR t.body LIKE ?)';
            $word = poket_db_replaceLikeWord($word);
            $params[] = '%'.$word.'%';
            $params[] = '%'.$word.'%';
        }
    }

    $sql .= ' ORDER BY t.r_datetime DESC';

    $list = pocket_db_getAllPage($sql, $page, $page_size, $params);
    foreach ($list as $key => $item)
    {
        $list[$key]['comment_count'] = db_get_topic_comments_count($item['topic_id']);
    }
    return $list;
}

/**
 *  キーワード検索のトピック件数の取得
 *
 *  @access public
 *  @param  string  $keyword            キーワード
 */
function db_get_topics_count4keyword($keyword)
{
    $sql = 'SELECT count(topic_id)' .
            ' FROM pbb_topics' .
            ' WHERE 1 = 1';

    if ($keyword)
    {
        $keyword = str_replace('　', ' ', $keyword);
        $word_list = explode(' ', $keyword);
        foreach ($word_list as $word)
        {
            $sql .= ' AND (t.subject LIKE ? OR t.body LIKE ?)';
            $word = poket_db_replaceLikeWord($word);
            $params[] = '%'.$word.'%';
            $params[] = '%'.$word.'%';
        }
    }

    return pocket_db_getOne($sql, $params);
}

/**
 *  トピックのパスワード認証
 *
 *  @access public
 *  @param  integer $user_id            トピックID
 *  @param  string  $password           パスワード
 */
function db_auth_topic4author_password($topic_id, $author_password)
{
    $sql = 'SELECT topic_id' .
            ' FROM pbb_topics' .
            ' WHERE topic_id = ?' .
            ' AND author_password = ?';
    $params = array(
        intval($topic_id),
        $author_password
    );
    return (bool)pocket_db_getOne($sql, $params);
}

/**
 *  トピック情報の登録
 *
 *  @access public
 *  @param  string  $author_name        名前
 *  @param  string  $author_email       E-Mail
 *  @param  string  $subject            タイトル
 *  @param  string  $body               本文
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $allow_comment      コメント削除許可フラグ
 *  @param  string  $author_password    暗証番号
 */
function db_insert_topic($author_name, $author_email, $subject, $body, $category_id, $allow_comment, $author_password)
{
    $data = array(
        'author_name' => $author_name,
        'author_email' => $author_email,
        'subject' => $subject,
        'body' => $body,
        'category_id' => intval($category_id),
        'allow_comment' => intval($allow_comment),
        'author_password' => $author_password,
        'r_datetime' => pocket_db_getNow(),
        'm_datetime' => pocket_db_getNow()
    );
    $insertId = pocket_db_insert('pbb_topics', $data, 'pbb_topics_topic_id_seq');

    $data = array(
        'category_id' => intval($category_id),
        'topic_id' => intval($insertId)
    );
    pocket_db_insert('pbb_category_topics', $data, 'pbb_category_topics_category_topic_id_seq');

    return $insertId;
}

/**
 *  トピック情報の更新
 *
 *  @access public
 *  @param  integer $topic_id           ID
 *  @param  string  $author_name        名前
 *  @param  string  $author_email       E-Mail
 *  @param  string  $subject            タイトル
 *  @param  string  $body               本文
 *  @param  integer $category_id        カテゴリID
 *  @param  integer $allow_comment      コメント削除許可フラグ
 */
function db_update_topic($topic_id, $author_name, $author_email, $subject, $body, $category_id, $allow_comment)
{
    $data = array(
        'author_name' => $author_name,
        'author_email' => $author_email,
        'subject' => $subject,
        'body' => $body,
        'category_id' => intval($category_id),
        'allow_comment' => intval($allow_comment),
        'm_datetime' => pocket_db_getNow()
    );
    $where = array(
        'topic_id' => intval($topic_id)
    );
    if (!pocket_db_update('pbb_topics', $data, $where)) {return false;}

    $data = array(
        'category_id' => intval($category_id)
    );
    $where = array(
        'topic_id' => intval($topic_id)
    );
    return pocket_db_update('pbb_category_topics', $data, $where);
}

/**
 *  トピック情報のカテゴリの更新
 *
 *  @access public
 *  @param  integer $category_id        カテゴリID
 */
function db_update_topics_category4default($category_id)
{
    $data = array(
        'category_id' => 1
    );
    $where = array(
        'category_id' => intval($category_id)
    );
    if (!pocket_db_update('pbb_topics', $data, $where)) {return false;}

    $data = array(
        'category_id' => 1
    );
    $where = array(
        'category_id' => intval($category_id)
    );
    return pocket_db_update('pbb_category_topics', $data, $where);
}

/**
 *  トピック情報の削除
 *
 *  @access public
 *  @param  integer $topic_id            ID
 */
function db_delete_topic($topic_id)
{
    if (!pocket_db_delete('pbb_topics', 'topic_id', intval($topic_id))) {return false;}
    if (!pocket_db_delete('pbb_category_topics', 'topic_id', intval($topic_id))) {return false;}
    return pocket_db_delete('pbb_topic_comments', 'topic_id', intval($topic_id));
}

?>
